import java.util.ArrayList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.experimental.chart.swt.ChartComposite;

public class GUI {
	XBenchMatch xbm;
	Display display;
	Shell shell;
	ScrolledComposite scrollComposite;
	Composite mainComposite, schemasChoiceComposite;
	Combo matchersCombo;
	String[] matchersList;
	MessageBox alertMsg;
	SchemaContentStructure scc;
	ArrayList <Label>featuresLabel;
	ArrayList <Text>featuresText;
	GridData gridDataSpanH2;
	boolean generateMediatedSchema, generateMappingsSet;
	Label precisionLabel, recallLabel, fscoreLabel, schemaProximityLabel, overallLabel, biggestSubtreeLabel, structuralOverlapLabel, structuralProximityLabel;
	Label inputFileLabel, idealFileLabel;
	String inputFilePath, idealFilePath;
	ChartComposite graph;
	MenuItem matcherFileMenuItem, executeBenchmarkMenuItem;
	final int MES_STRUCT_PROX = 0;
	final int MES_FSCORE = 1;
	final int MES_PRECISION = 2;
	final int MES_RECALL = 3;
	final int MES_OVERALL = 4;
	final int MES_SCHEMA_PROX = 5;
	
	public GUI(String[] matchList, XBenchMatch xbenchm)
	{
		matchersList = matchList;
		xbm = xbenchm;
		featuresLabel = new ArrayList<Label>();
		featuresText = new ArrayList<Text>();
		generateMediatedSchema = false;
		generateMappingsSet = false;
		scc = new SchemaContentStructure();
		gridDataSpanH2 = new GridData();
		gridDataSpanH2.horizontalAlignment = GridData.FILL;
		gridDataSpanH2.horizontalSpan = 2;
	}
	
	public String createInputDialog(String text, String mess, String prevInput) {
		InputDialog id = new InputDialog(shell, SWT.NONE, text, mess);
		id.setMessage(text);
		id.setInput(prevInput);
		return id.open();
	}
	
	public void displayGraph(BenchmarkResult [][]tabBr, ArrayList<String> scenarii, ArrayList<ArrayList<String>> matchers, int measure) {
		Shell sh = new Shell(display);
        sh.setLayout(new FillLayout());
        DefaultCategoryDataset data = new DefaultCategoryDataset();
		for(int i = 0; i < scenarii.size(); i++) {
			for(int j = 0; j < matchers.get(i).size(); j++) {
				if(measure == MES_STRUCT_PROX) data.addValue(tabBr[i][j].structuralProximity, matchers.get(i).get(j), scenarii.get(i));
				else if(measure == MES_OVERALL) data.addValue(tabBr[i][j].overall, matchers.get(i).get(j), scenarii.get(i));
				else if(measure == MES_RECALL) data.addValue(tabBr[i][j].recall, matchers.get(i).get(j), scenarii.get(i));
				else if(measure == MES_PRECISION) data.addValue(tabBr[i][j].precision, matchers.get(i).get(j), scenarii.get(i));
				else if(measure == MES_SCHEMA_PROX) data.addValue(tabBr[i][j].schemaProximity, matchers.get(i).get(j), scenarii.get(i));
				else data.addValue(tabBr[i][j].fscore, matchers.get(i).get(j), scenarii.get(i));
			}
		}
		String mesure ="";
		if(measure == MES_STRUCT_PROX) mesure = "Structural Proximity";
		else if(measure == MES_OVERALL) mesure = "Overall";
		else if(measure == MES_RECALL) mesure = "Recall";
		else if(measure == MES_PRECISION) mesure = "Precision";
		else if(measure == MES_SCHEMA_PROX) mesure = "Schema Proximity";
		else  mesure = "F-score";
        JFreeChart chart = ChartFactory.createBarChart3D("Comparison of different matching tools on the matching quality", "Scenarii", mesure, data, PlotOrientation.VERTICAL, true, true, false);
        CategoryPlot plot = (CategoryPlot)chart.getPlot();
        (plot.getRangeAxis()).setLowerBound(0.0);
        (plot.getRangeAxis()).setUpperBound(1.0);
        graph = new ChartComposite(sh, SWT.NONE, chart);
        graph.pack(true);
        sh.open();
    }
	
	public void displayGraphMappings(BenchmarkResult []tabBr) {
		Shell sh = new Shell(display);
        sh.setLayout(new FillLayout());
        XYSeriesCollection dataset = new XYSeriesCollection();
        XYSeries seriePrec = new XYSeries("Precision");
        XYSeries serieRec = new XYSeries("Recall");
        XYSeries serieFsc = new XYSeries("F-score");
        XYSeries serieOve = new XYSeries("Overall");
		for(int i=0;i<tabBr.length;i++) {
			System.out.println(tabBr[i].nbElementsMatcherSchema+" mappings, "+tabBr[i].precision+" prec,  "+tabBr[i].recall+"rec,  "+tabBr[i].fscore+" fsco,  "+tabBr[i].overall+" overal..");
			seriePrec.add(tabBr[i].nbElementsMatcherSchema, tabBr[i].precision); //tabBr[i].nbCommonElementsInSubtrees
			serieRec.add(tabBr[i].nbElementsMatcherSchema, tabBr[i].recall);
			serieFsc.add(tabBr[i].nbElementsMatcherSchema, tabBr[i].fscore);
			serieOve.add(tabBr[i].nbElementsMatcherSchema, tabBr[i].overall);
		}
		dataset.addSeries(seriePrec);
		dataset.addSeries(serieRec);
		dataset.addSeries(serieFsc);
		dataset.addSeries(serieOve);
        JFreeChart chart = ChartFactory.createXYLineChart("Quality measures", "Number of mappings", "Measures values", dataset, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = chart.getXYPlot();
        plot.setRenderer(new StandardXYItemRenderer(StandardXYItemRenderer.SHAPES_AND_LINES));
        graph = new ChartComposite(sh, SWT.NONE, chart);
        graph.pack(true);
        sh.open();
        updateMainComposite();
	}
	
	public void displayGraph(BenchmarkResult []tabBr) {
		//clearMainComposite();
		Shell sh = new Shell(display);
        sh.setLayout(new FillLayout());
        XYSeriesCollection dataset = new XYSeriesCollection();
        XYSeries seriePrec = new XYSeries("Precision");
        XYSeries serieRec = new XYSeries("Recall");
        XYSeries serieFsc = new XYSeries("F-score");
        XYSeries serieOve = new XYSeries("Overall");
		for(int i=0;i<tabBr.length;i++) {
			System.out.println(tabBr[i].nbElementsMatcherSchema+" elem schema, "+tabBr[i].precision+" prec,  "+tabBr[i].recall+"rec,  "+tabBr[i].fscore+" fsco,  "+tabBr[i].overall+" overal..");
			//System.out.println(tabBr[i].nbElementsIdealSchema+"in ideal ;;; "+tabBr[i].nbElementsMatcherSchema+"nb elements in matcher");
			seriePrec.add(tabBr[i].nbElementsIdealSchema, tabBr[i].precision); //tabBr[i].nbCommonElementsInSubtrees
			serieRec.add(tabBr[i].nbElementsIdealSchema, tabBr[i].recall);
			serieFsc.add(tabBr[i].nbElementsIdealSchema, tabBr[i].fscore);
			serieOve.add(tabBr[i].nbElementsIdealSchema, tabBr[i].overall);
		}
		dataset.addSeries(seriePrec);
		dataset.addSeries(serieRec);
		dataset.addSeries(serieFsc);
		dataset.addSeries(serieOve);
        JFreeChart chart = ChartFactory.createXYLineChart("Quality measures", "Number of schema elements", "Measures values", dataset, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = chart.getXYPlot();
        plot.setRenderer(new StandardXYItemRenderer(StandardXYItemRenderer.SHAPES_AND_LINES));
        graph = new ChartComposite(sh, SWT.NONE, chart);
        graph.pack(true);
        sh.open();
        updateMainComposite();
	}
	
	public void displayGraph2(BenchmarkResult []tabBr) {
		Shell sh = new Shell(display);
        sh.setLayout(new FillLayout());
        XYSeriesCollection dataset = new XYSeriesCollection();
        XYSeries seriePrec = new XYSeries("Structural overlap");
        XYSeries serieRec = new XYSeries("Structural proximity");
        XYSeries serieFsc = new XYSeries("Schema proximity");
		for(int i=0;i<tabBr.length;i++) {
			System.out.println(tabBr[i].nbElementsMatcherSchema+" elem schema, "+tabBr[i].precision+" prec,  "+tabBr[i].recall+"rec,  "+tabBr[i].fscore+" fsco,  "+tabBr[i].overall+" overal..");
			//System.out.println(tabBr[i].nbElementsIdealSchema+"in ideal ;;; "+tabBr[i].nbElementsMatcherSchema+"nb elements in matcher");
			seriePrec.add(tabBr[i].nbElementsIdealSchema, tabBr[i].structuralOverlap); //tabBr[i].nbCommonElementsInSubtrees
			serieRec.add(tabBr[i].nbElementsIdealSchema, tabBr[i].structuralProximity);
			serieFsc.add(tabBr[i].nbElementsIdealSchema, tabBr[i].schemaProximity);
		}
		dataset.addSeries(seriePrec);
		dataset.addSeries(serieRec);
		dataset.addSeries(serieFsc);
        JFreeChart chart = ChartFactory.createXYLineChart("Structural quality measures", "Number of schema elements", "Measures values", dataset, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = chart.getXYPlot();
        plot.setRenderer(new StandardXYItemRenderer(StandardXYItemRenderer.SHAPES_AND_LINES));
        graph = new ChartComposite(sh, SWT.NONE, chart);
        graph.pack(true);
        sh.open();
        updateMainComposite();
	}	
	
	public int alert(String text, int style) {
		alertMsg = new MessageBox(shell, style); 
		alertMsg.setMessage(text);
		return alertMsg.open();
	}
	
	public String getChosenFile(String title) {
        FileDialog fd = new FileDialog(shell, SWT.OPEN | SWT.SINGLE);
        fd.setText(title);
        return fd.open();
	}

	public void clearMainComposite() {
		Widget[] childrens = mainComposite.getChildren();
		for (int i = 0; i < childrens.length; i++)
		         childrens[i].dispose();
	}
	
	public void updateMainComposite() {
		//mainComposite.pack(true);
		mainComposite.layout(true);
		//mainComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		scrollComposite.setMinSize(mainComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		scrollComposite.setContent(mainComposite);
		//scrollComposite.layout();
	}
	
	public void displayReport(String theSubItemText) {
		String filePath = "default/" + theSubItemText + ".xml";
		xbm.updateFeatures(filePath, scc);
		clearMainComposite();
		Label tempLabel = new Label(mainComposite, SWT.NONE);
		tempLabel.setLayoutData(gridDataSpanH2);
		tempLabel = new Label(mainComposite, SWT.NONE);
		tempLabel.setText("FEATURES FOR "+theSubItemText);
		tempLabel.setLayoutData(gridDataSpanH2);
		tempLabel = new Label(mainComposite, SWT.HORIZONTAL | SWT.SEPARATOR);
		tempLabel.setLayoutData(gridDataSpanH2);
		for(int i=0; i<scc.nodesLabels.size();i++) {
			tempLabel = new Label(mainComposite, SWT.SINGLE);
			tempLabel.setText(scc.nodesLabels.get(i));
			Text tempText = new Text(mainComposite, SWT.SINGLE);
			tempText.setText(scc.nodesContent.get(i));
		}
		updateMainComposite();
	}
	
	public String selectFile(String text) {
        FileDialog fd = new FileDialog(shell, SWT.OPEN | SWT.SINGLE);
        fd.setText(text);
        return fd.open();
	}
	
	public void displayBenchMarkResult(String s, BenchmarkResult br)
	{
		Label tempLabel = new Label(mainComposite, SWT.NONE);
		tempLabel.setLayoutData(gridDataSpanH2);
		tempLabel = new Label(mainComposite, SWT.NONE);
		tempLabel.setText("BENCHMARK RESULTS FOR "+s);
		tempLabel.setLayoutData(gridDataSpanH2);
		tempLabel = new Label(mainComposite, SWT.HORIZONTAL | SWT.SEPARATOR);
		tempLabel.setLayoutData(gridDataSpanH2);
		precisionLabel = new Label(mainComposite, SWT.NONE);
		recallLabel = new Label(mainComposite, SWT.NONE);
		fscoreLabel = new Label(mainComposite, SWT.NONE);
		overallLabel = new Label(mainComposite, SWT.NONE);
		biggestSubtreeLabel = new Label(mainComposite, SWT.NONE);
		structuralOverlapLabel = new Label(mainComposite, SWT.NONE);
		structuralProximityLabel = new Label(mainComposite, SWT.NONE);
		schemaProximityLabel = new Label(mainComposite, SWT.NONE);
		precisionLabel.setText("Precision : "+br.precision);
		recallLabel.setText("Recall : "+br.recall);
		fscoreLabel.setText("F-score : "+br.fscore);
		overallLabel.setText("Overall : "+br.overall);
		biggestSubtreeLabel.setText("Biggest subtree size : "+br.biggestSubstreeSize);
		structuralOverlapLabel.setText("Structural overlap : "+br.structuralOverlap);
		structuralProximityLabel.setText("Structural proximity : "+br.structuralProximity);
		schemaProximityLabel.setText("Schema proximity : "+br.schemaProximity);
	}
	
	public String getFileNameFromPath(String s) {
		String ret = "";
		int ind = s.lastIndexOf("/");
		ret = s.substring(ind+1, s.length());
		return ret;
	}
	
	public void initGUI()
	{
		display = new Display();
		shell = new Shell(display);
		shell.setText("XBenchMatch");
		shell.setLayout(new FillLayout());
		scrollComposite = new ScrolledComposite(shell, SWT.V_SCROLL);
		mainComposite = new Composite(scrollComposite, SWT.NONE);
		scrollComposite.setExpandVertical(true);
		scrollComposite.setExpandHorizontal(true);
		//scrollComposite.setLayout(new FillLayout());
		GridLayout mainLayout = new GridLayout(2, true);
		mainLayout.horizontalSpacing = 20;
		mainLayout.verticalSpacing = 20;
		mainComposite.setLayout(mainLayout);
		// location of the shell in the middle
		Monitor primary = display.getPrimaryMonitor();
		Rectangle bounds = primary.getBounds ();
		shell.setBounds(0, 0, bounds.width, bounds.height);
		//mainComposite.setBounds(0, 0, bounds.width, bounds.height);
		
		// shell menu
		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar (menu);
		MenuItem menuitem = new MenuItem(menu, SWT.CASCADE);
		menuitem.setText("&File");
		Menu submenu = new Menu(shell, SWT.DROP_DOWN);
		menuitem.setMenu(submenu);
		MenuItem subitem = new MenuItem(submenu, SWT.DROP_DOWN);
		subitem.setText("Save Benchmark");
		subitem.setEnabled(false);
		subitem = new MenuItem(submenu, SWT.DROP_DOWN);
		subitem.setText("Exit");
		subitem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
				System.exit(0);
			}
		});
		menuitem = new MenuItem(menu, SWT.PUSH);
		menuitem.setText("&Run default benchmark");
		menuitem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
				xbm.defaultBench();
			}
		});
		menuitem = new MenuItem(menu, SWT.CASCADE);
		menuitem.setText("Run &specific scenario");
		//menuitem.setEnabled(false);
		submenu = new Menu(shell, SWT.DROP_DOWN);
		menuitem.setMenu(submenu);
		subitem = new MenuItem(submenu, SWT.DROP_DOWN);
		subitem.setText("Select the ideal file");
		subitem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
		        idealFilePath = selectFile("Select the ideal file");
				if(idealFilePath!=null) {
					((MenuItem)e.widget).setText("Selected ideal file: "+getFileNameFromPath(idealFilePath));
					matcherFileMenuItem.setEnabled(true);
				}
			}
		});
		matcherFileMenuItem = new MenuItem(submenu, SWT.DROP_DOWN);
		matcherFileMenuItem.setText("Select the matcher's file");
		//matcherFileMenuItem.setEnabled(false);
		matcherFileMenuItem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
		        inputFilePath = selectFile("Select the matcher's file");
		        if(inputFilePath!=null) {
		        	((MenuItem)e.widget).setText("Selected matcher's file: "+getFileNameFromPath(inputFilePath));
		        	executeBenchmarkMenuItem.setEnabled(true);
		        }
			}
		});
		executeBenchmarkMenuItem = new MenuItem(submenu, SWT.DROP_DOWN);
		executeBenchmarkMenuItem.setText("Execute benchmark");
		executeBenchmarkMenuItem.setEnabled(false);
		executeBenchmarkMenuItem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
		    	  if(idealFilePath == null || inputFilePath==null)
			    		 alert("You must choose both an ideal file and the one generated by a matching tool.", SWT.OK | SWT.ICON_ERROR);
		    	  else xbm.benchFiles(idealFilePath, inputFilePath, true);
			}
		});
		menuitem = new MenuItem(menu, SWT.DROP_DOWN);
		menuitem.setText("&Compare matching tools");
		//menuitem.setEnabled(false);
		menuitem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
				clearMainComposite();
				xbm.compareMatchers();
			}
		});
		menuitem = new MenuItem(menu, SWT.CASCADE);
		menuitem.setText("&Matching tools features");
		submenu = new Menu(shell, SWT.DROP_DOWN);
		menuitem.setMenu(submenu);
		for(int i=0; i<matchersList.length;i++)
		{
			subitem = new MenuItem(submenu, SWT.DROP_DOWN);
			subitem.setText(matchersList[i].substring(0, matchersList[i].length()-4));
			subitem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				System.out.println("Combo default selected (overrides default button)");}
			public void widgetSelected(SelectionEvent e) {
				String theSubItemText = ((MenuItem)e.widget).getText();
				displayReport(theSubItemText);
			}
		});
		}
		menuitem = new MenuItem(menu, SWT.CASCADE);
		menuitem.setText("&Tools");
		submenu = new Menu(shell, SWT.DROP_DOWN);
		menuitem.setMenu(submenu);
		subitem = new MenuItem(submenu, SWT.DROP_DOWN);
		subitem.setText("Convert an ascii-tree to xsd");
		subitem.addSelectionListener(new SelectionListener() {
		public void widgetDefaultSelected(SelectionEvent e) {
			System.out.println("Combo default selected (overrides default button)");}
		public void widgetSelected(SelectionEvent e) {
			String asciiTreeFile = getChosenFile("Choose the ascii-tree file to convert");
			if(asciiTreeFile!=null) {
				boolean ret = xbm.convertAsciiTreeToXSD(asciiTreeFile);
				if(ret) alert("Ascii-tree file has been converted to "+asciiTreeFile+".xsd !", 0);
				else alert("Error while trying to convert "+asciiTreeFile+" !", 1);
			}
		}
		});

		scrollComposite.setContent(mainComposite);
		scrollComposite.setMinSize(mainComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch ()) display.sleep ();
		}
		display.dispose();
	}	
}