import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

public class MappingBenchmark {
	public File idealInputFile;
	public File matcherInputFile;
	public int nbIdealMappings;
	public Mapping[] tabIdealMappings;
	
	public MappingBenchmark(String idealFile, String matcherFile)
	{
		idealInputFile = new File(idealFile);
		matcherInputFile = new File(matcherFile);
		nbIdealMappings = 0;
	}
	
	public boolean init()
	{
		try {
			BufferedReader in  = new BufferedReader(new FileReader(idealInputFile));
			while(in.ready())
			{
				String s = in.readLine();
				nbIdealMappings++;
			}
			tabIdealMappings = new Mapping[nbIdealMappings];
			in  = new BufferedReader(new FileReader(idealInputFile));
			int counter = 0;
			while(in.ready())
			{
				String s = in.readLine();
				String[] tok = s.split(";");
				Mapping m = null;
				if(tok.length == 2) m = new Mapping(tok[0].trim(), tok[1].trim(), "");
				else if(tok.length > 2) m = new Mapping(tok[0].trim(), tok[1].trim(), tok[2].trim());
				m.printMapping();
				if(m != null)
				{
					tabIdealMappings[counter] = m;
					counter++;
				}
			}
		}
		catch(Exception e) {
			System.out.println("Error: problem with ideal file ["+e.toString()+"].");
			return false;
		}
		return true;
	}
	
	public BenchmarkResult bench()
	{
		System.out.println("=== in second matcher file ===");
		BenchmarkResult br = new BenchmarkResult();
		int nbDiscoveredMappings = 0;
		int nbRelevantMappings = 0;
		try {
			BufferedReader in  = new BufferedReader(new FileReader(matcherInputFile));
			while(in.ready())
			{
				String s = in.readLine();
				if(!s.equals("")) {
					nbDiscoveredMappings++;
					String[] tok = s.split(";");
					Mapping m = null;
					if(tok.length == 2) m = new Mapping(tok[0].trim(), tok[1].trim(), "");
					else if(tok.length > 2) m = new Mapping(tok[0].trim(), tok[1].trim(), tok[2].trim());
					if(m != null)
					{
						m.printMapping();
						for(int i=0; i<tabIdealMappings.length;i++)
							if(m.compareMappings(tabIdealMappings[i]) == true)
							{
								m.printMapping();
								nbRelevantMappings++;
							}
					}
				}
			}
			br.precision = (double) nbRelevantMappings / (double) nbDiscoveredMappings;
			br.recall = (double) nbRelevantMappings / (double) nbIdealMappings;
			br.nbElementsMatcherSchema = nbDiscoveredMappings;
			br.calculateFScore();
			br.calculateOverall();
		}
		catch(Exception e) {
			System.out.println("Error: problem with matcher file ["+e.toString()+"].");
			return null;
		}
		return br;
	}
}
