import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.swing.tree.DefaultMutableTreeNode;

public class SchemaBenchmark {
	public DefaultMutableTreeNode rootIdealSchema, rootMatcherSchema;
	public String idealSchemaPath, matcherSchemaPath;
	public File matcherSchemaFile;
	private ArrayList<DefaultMutableTreeNode> commonSubtrees;
	
	public SchemaBenchmark(String idPath, String matchPath) {
		idealSchemaPath = idPath;
		matcherSchemaPath = matchPath;
		matcherSchemaFile = new File(matchPath);
	}
	
	public boolean init() {
		XMLParser xmlp = new XMLParser();
		rootIdealSchema = xmlp.startParsingMediatedSchema(idealSchemaPath);
		rootMatcherSchema = xmlp.startParsingMediatedSchema(matcherSchemaPath);
		if(rootIdealSchema !=null && rootMatcherSchema != null) return true;
		else return false;
	}
	
	int checkChildren(DefaultMutableTreeNode e1, DefaultMutableTreeNode e2) {
		int ret = 0;
		Enumeration e1Children = e1.children();
		Enumeration e2Children;
		DefaultMutableTreeNode tempNodeE1, tempNodeE2;
		
		while(e1Children.hasMoreElements()) {
			e2Children = e2.children();
			tempNodeE1 = (DefaultMutableTreeNode) e1Children.nextElement();
			while(e2Children.hasMoreElements()) {
				tempNodeE2 = (DefaultMutableTreeNode) e2Children.nextElement();
				if(tempNodeE1.getUserObject().equals(tempNodeE2.getUserObject()))
					ret  += 1 +checkChildren(tempNodeE1, tempNodeE2);
			}
		}
		return ret;
	}
	
	int findBiggestSubtree() {
		Enumeration eIdeal;
		Enumeration eMatcher = rootMatcherSchema.breadthFirstEnumeration();
		DefaultMutableTreeNode tempNodeMatcher, tempNodeIdeal;
		int bestSubtreeSize=0, tempSubtreeSize;
		
		while(eMatcher.hasMoreElements()) {
			eIdeal = rootIdealSchema.breadthFirstEnumeration();
			tempNodeMatcher = (DefaultMutableTreeNode) eMatcher.nextElement();
			while(eIdeal.hasMoreElements()) {
				tempSubtreeSize = 0;
				tempNodeIdeal = (DefaultMutableTreeNode) eIdeal.nextElement();
				if(tempNodeIdeal.getUserObject().equals(tempNodeMatcher.getUserObject())) {
					// comparing children
					tempSubtreeSize = 1 +checkChildren(tempNodeIdeal, tempNodeMatcher);
					if(tempSubtreeSize > bestSubtreeSize) bestSubtreeSize = tempSubtreeSize;
				}
			}
		}
		return bestSubtreeSize;
	}
	
	public boolean nodeInTheSubtree(String s, DefaultMutableTreeNode n) {
		if(n.getUserObject().toString().equals(s)) return true;
		else {
			for(int j=0; j<n.getChildCount();j++)
				if(nodeInTheSubtree(s, (DefaultMutableTreeNode)n.getChildAt(j))) return true;
		}
		return false;
	}
	
	public boolean nodeAlreadyInCommonSubtrees(String s)
	{
		DefaultMutableTreeNode[] tabCommonSubtrees = new DefaultMutableTreeNode[commonSubtrees.size()];
		tabCommonSubtrees = commonSubtrees.toArray(tabCommonSubtrees);
		for(int i=0; i< tabCommonSubtrees.length;i++)
			if(nodeInTheSubtree(s, tabCommonSubtrees[i])) return true;
		return false;
	}
	
	public DefaultMutableTreeNode buildSubtree(DefaultMutableTreeNode nodeM, DefaultMutableTreeNode nodeI) {
		DefaultMutableTreeNode nodeRet = new DefaultMutableTreeNode(nodeI.getUserObject());
		for(int i=0; i<nodeI.getChildCount();i++) {
			DefaultMutableTreeNode childI = (DefaultMutableTreeNode) nodeI.getChildAt(i);
			for(int m=0; m<nodeM.getChildCount();m++) {
				DefaultMutableTreeNode childM = (DefaultMutableTreeNode) nodeM.getChildAt(m);
				if(childI.getUserObject().equals(childM.getUserObject()) && !nodeAlreadyInCommonSubtrees(childI.getUserObject().toString()))
					nodeRet.add(buildSubtree(childI, childM));
			}
		}
		return nodeRet;
	}
	
	public int getNbCommonElementsInSubtrees() {
		int ret = 0;
		DefaultMutableTreeNode[] tabCommonSubtrees = new DefaultMutableTreeNode[commonSubtrees.size()];
		tabCommonSubtrees = commonSubtrees.toArray(tabCommonSubtrees);
		for(int i=0; i< tabCommonSubtrees.length;i++) {
			Enumeration tempE = tabCommonSubtrees[i].breadthFirstEnumeration();
			while(tempE.hasMoreElements()) {
				ret++;
				tempE.nextElement();
			}
		}
		return ret;
	}
	
	/*public int calculateDepth(DefaultMutableTreeNode n) {
		if(n.getParent()==null) return 1;
		else return 1+calculateDepth((DefaultMutableTreeNode)n.getParent());
	}*/
	
	public BenchmarkResult compareTrees()
	{
		BenchmarkResult br = new BenchmarkResult();
		int nbDiscoveredElements = 0; // nb elements in matcher mediated schema
		int nbIdealElements = 0; // nb elements in ideal mediated schema
		int nbCommonElements = 0; // common elements to both schemas
		int nbSubtrees = 0;
		int nbCommonElementsInSubtrees = 0;
		DefaultMutableTreeNode tempNodeMatcher, tempNodeIdeal;
		Enumeration eIdeal = rootIdealSchema.breadthFirstEnumeration();
		Enumeration eMatcher = rootMatcherSchema.breadthFirstEnumeration();
		
		//int maxDepth=0, tempDepth;
		// count the number of elements in ideal mediated schema
		while(eIdeal.hasMoreElements()) {
			nbIdealElements++;
			//DefaultMutableTreeNode temp = (DefaultMutableTreeNode)eIdeal.nextElement();
			//tempDepth=calculateDepth(temp);
			//if(tempDepth>maxDepth) maxDepth=tempDepth;
			eIdeal.nextElement();
		}
		//System.out.println("max depth IDEAL = "+maxDepth);
		
		// find common elemnts
		while(eMatcher.hasMoreElements()) {
			eIdeal = rootIdealSchema.breadthFirstEnumeration();
			tempNodeMatcher = (DefaultMutableTreeNode) eMatcher.nextElement();
			nbDiscoveredElements++;
			while(eIdeal.hasMoreElements()) {
				tempNodeIdeal = (DefaultMutableTreeNode) eIdeal.nextElement();
				if(tempNodeIdeal.getUserObject().equals(tempNodeMatcher.getUserObject())) {
					nbCommonElements++;
					break;
				}
			}			
		}
		
		// find all disjoint common subtrees of size > 2
		commonSubtrees = new ArrayList<DefaultMutableTreeNode>();
		eIdeal = rootIdealSchema.breadthFirstEnumeration();
		while(eIdeal.hasMoreElements()) {
			eMatcher = rootMatcherSchema.breadthFirstEnumeration();
			tempNodeIdeal = (DefaultMutableTreeNode) eIdeal.nextElement();
			while(eMatcher.hasMoreElements()) {
				tempNodeMatcher = (DefaultMutableTreeNode) eMatcher.nextElement();
				if(tempNodeIdeal.getUserObject().equals(tempNodeMatcher.getUserObject()) && !nodeAlreadyInCommonSubtrees(tempNodeMatcher.getUserObject().toString())) {
					DefaultMutableTreeNode tempSubtree = buildSubtree(tempNodeMatcher, tempNodeIdeal);
					if(!tempSubtree.isLeaf()) {
						commonSubtrees.add(tempSubtree);
						break;
					}
				}
			}
		}

		if(nbCommonElements > nbIdealElements) nbCommonElements = nbIdealElements; // to avoid repetitive element to result in a recall > 1
		br.precision = (double) nbCommonElements / (double) nbDiscoveredElements;
		br.recall = (double) nbCommonElements / (double) nbIdealElements;
		br.calculateFScore();
		br.calculateOverall();
		br.nbElementsIdealSchema = nbIdealElements;
		br.nbElementsMatcherSchema = nbDiscoveredElements;
		br.biggestSubstreeSize = findBiggestSubtree();
		nbSubtrees = commonSubtrees.size();
		nbCommonElementsInSubtrees = getNbCommonElementsInSubtrees();
		if(nbCommonElementsInSubtrees > nbIdealElements) nbCommonElementsInSubtrees = nbIdealElements;
		br.nbCommonElementsInSubtrees = nbCommonElementsInSubtrees;
		br.structuralOverlap = (double) nbCommonElementsInSubtrees / (double) nbIdealElements;
		br.structuralProximity = (double) nbCommonElementsInSubtrees / ((double) nbIdealElements * (Math.sqrt((double) nbSubtrees + (nbIdealElements - nbCommonElementsInSubtrees))));
		br.schemaProximity = Math.exp(((1.0/nbSubtrees) * ((double)(nbCommonElements - (nbIdealElements-nbCommonElements) - (nbDiscoveredElements-nbCommonElements))/(double)nbIdealElements))-1);
		//if(br.schemaProximity<-1.0) br.schemaProximity = -1.0;
		return br;
	}
}
