import java.io.File;
import java.util.ArrayList;

public class XBenchMatch {
	public String[] matchersList;
	public BenchmarkResult br;
	static GUI gui;
	static int NB_SCENARIOS_DEFAULT_BENCHMARK = 4;
	static String DEFAULT_BENCHMARK_DIRECTORY = "defaultbenchmark";
	static String DEFAULT_DIRECTORY = "default";
	
	public void exitWithError(String err)
	{
		System.out.println(err);
		System.exit(1);
	}
	
	public XBenchMatch()
	{
		// checking default directory
		File repDefault = new File(DEFAULT_DIRECTORY);
		if(!repDefault.exists())
			exitWithError("Default directory does not exist. Application corrupted, you have to reinstall it.");
		matchersList = repDefault.list();
	}
	
	public void compareMatchers() {
		ArrayList<ArrayList<String>> matchers = new ArrayList<ArrayList<String>>();
		ArrayList<String> scenarii = new ArrayList<String>();
		ArrayList<ArrayList<String>> matchersFile = new ArrayList<ArrayList<String>>();
		ArrayList<String> scenariiFile = new ArrayList<String>();
		ArrayList<String> temp;
		String scenario = "", scenarioFile;
		int nbScenarii = 0;
		while(scenario!=null) {
			scenario = gui.createInputDialog("Scenario name required", "Please enter the name of the scenario and validate, or press finish:", "");
			if(scenario!=null) {
				scenarioFile = gui.selectFile("Select the ideal file for scenario "+scenario);
				if(scenarioFile!=null) {
					scenarii.add(scenario);
					scenariiFile.add(scenarioFile);
					String matcher = "", matcherFile;
					int cnt = 0;
					temp = new ArrayList<String>();
					matchers.add(temp);
					while(matcher!=null) {
						String previousInput = null;
						//System.out.println("nbscenar="+nbScenarii+"   cnt="+cnt+"     vectsize="+matchers.get(0).size());
						//if(nbScenarii>=1 && cnt<matchers.get(0).size()) previousInput = matchers.get(0).get(cnt);
						//System.out.println("et le prvinput= "+previousInput);
						matcher = gui.createInputDialog("Matching tool name required", "Please enter the name of the matching tool and validate, or press finish:", previousInput);
						if(matcher != null) {
							matcherFile = gui.selectFile("Select the file generated by matcher "+matcher+" for scenario "+scenario);
							if(matcherFile!=null) {
								matchers.get(nbScenarii).add(matcher);
								temp = new ArrayList<String>();
								matchersFile.add(temp);
								matchersFile.get(nbScenarii).add(matcherFile);
							}
						}
						cnt++;
					}
					nbScenarii++;
				}
			}
		}
		if(scenarii.size()<1) gui.alert("At least one scenario is required to evaluate the matching tools ! Comparison is cancelled", 0);
		else {
			BenchmarkResult [][]tabBr = new BenchmarkResult[scenariiFile.size()][matchersFile.size()];
			for(int i = 0; i < scenariiFile.size(); i++) {
				for(int j = 0; j < matchersFile.get(i).size(); j++) {
					benchFiles(scenariiFile.get(i), matchersFile.get(i).get(j), false);
					tabBr[i][j] = br;
				}
			}
			gui.displayGraph(tabBr, scenarii, matchers, gui.MES_FSCORE);
			gui.displayGraph(tabBr, scenarii, matchers, gui.MES_PRECISION);
			gui.displayGraph(tabBr, scenarii, matchers, gui.MES_RECALL);
			//gui.displayGraph(tabBr, scenarii, matchers, gui.MES_OVERALL);
			//gui.displayGraph(tabBr, scenarii, matchers, gui.MES_STRUCT_PROX);
			gui.displayGraph(tabBr, scenarii, matchers, gui.MES_SCHEMA_PROX);
		}
	}
	
	public boolean convertAsciiTreeToXSD(String asciiTreePath) {
		ConvertAsciiTreeTool catt = new ConvertAsciiTreeTool(asciiTreePath);
		return catt.convert();
	}
	
	public void benchFiles(String idealSchemaPath, String matcherSchemaPath, boolean clearGui) {
		if(clearGui) gui.clearMainComposite();
		if(idealSchemaPath.substring(idealSchemaPath.length()-4, idealSchemaPath.length()).equalsIgnoreCase(".xml") || idealSchemaPath.substring(idealSchemaPath.length()-4, idealSchemaPath.length()).equalsIgnoreCase(".xsd"))
			benchSchemas(idealSchemaPath, matcherSchemaPath);
		else benchMappings(idealSchemaPath, matcherSchemaPath);
		gui.updateMainComposite();
	}
	
	public void benchSchemas(String idealSchemaPath, String matcherSchemaPath) {
		SchemaBenchmark sb = new SchemaBenchmark(idealSchemaPath, matcherSchemaPath);
		if(!sb.init()) {
			br = new BenchmarkResult();
			System.out.println("Schemas corrupted. Aborting schema benchmark.");
			gui.alert("One of the mediated schema is not valid !", 0);
		}
		else {
			br = sb.compareTrees();
			if(br == null) gui.alert("The mediated schemas are not valid XML schemas !", 0); 
			else gui.displayBenchMarkResult(sb.matcherSchemaFile.getName(), br);
		}
	}
	
	public void benchMappings(String idealMappingPath, String matcherMappingPath) {
		MappingBenchmark mb = new MappingBenchmark(idealMappingPath, matcherMappingPath);
		if(!mb.init()) {
			br = new BenchmarkResult();
			System.out.println("Set of mappings corrupted. Aborting schema benchmark.");
			gui.alert("Ideal set of mappings is not a valid file !", 0);
		}
		else {
			br = mb.bench();
			if(br == null) gui.alert("Matcher set of mappings is not a valid file !", 0); 
			else gui.displayBenchMarkResult(mb.matcherInputFile.getName(), br);
		}
	}
	
	public void defaultBench() {
		File repDefault = new File(DEFAULT_BENCHMARK_DIRECTORY);
		if(!repDefault.exists()) // checking default directory
			exitWithError("defaultbenchmark directory does not exist. Application corrupted, you have to reinstall it.");
		String pMPerson = gui.getChosenFile("Choose the matcher's file for PERSON schemas");
		String pMOrder = gui.getChosenFile("Choose the matcher's file for ORDER schemas");
		String pMUniversity = gui.getChosenFile("Choose the matcher's file for UNIVERSITY schemas");
		String pMBio = gui.getChosenFile("Choose the matcher's file for BIOLOGY schemas");
		
		if(pMPerson == null || pMOrder == null || pMUniversity == null || pMBio == null)
			gui.alert("One matcher's integrated schema needs to be chosen for each scenario !", 0);
		else {
			BenchmarkResult []tabBr = new BenchmarkResult[NB_SCENARIOS_DEFAULT_BENCHMARK];
			boolean mapp=false; // on utilise des mappings ou des schemas
			
			gui.clearMainComposite();
			if(pMPerson.substring(pMPerson.length()-4, pMPerson.length()).equalsIgnoreCase(".xml") || pMPerson.substring(pMPerson.length()-4, pMPerson.length()).equalsIgnoreCase(".xsd"))
				benchSchemas(repDefault+"/Personne_ideal_schema.xsd", pMPerson);
			else {benchMappings(repDefault+"/Personne_ideal_mappings.txt", pMPerson); mapp=true;}
			tabBr[0] = br;
			
			if(pMOrder.substring(pMOrder.length()-4, pMOrder.length()).equalsIgnoreCase(".xml") || pMOrder.substring(pMOrder.length()-4, pMOrder.length()).equalsIgnoreCase(".xsd"))
				benchSchemas(repDefault+"/Order_ideal_schema.xsd", pMOrder);
			else benchMappings(repDefault+"/Order_ideal_mappings.txt", pMOrder);
			tabBr[1] = br;
			
			if(pMUniversity.substring(pMUniversity.length()-4, pMUniversity.length()).equalsIgnoreCase(".xml") || pMUniversity.substring(pMUniversity.length()-4, pMUniversity.length()).equalsIgnoreCase(".xsd"))
				benchSchemas(repDefault+"/University_ideal_schema.xsd", pMUniversity);
			else benchMappings(repDefault+"/University_ideal_mappings.txt", pMUniversity);
			tabBr[2] = br;
			
			benchSchemas(repDefault+"/Biology_ideal_schema.xsd", pMBio);
			tabBr[3] = br;
	
			if(!mapp) {
				gui.displayGraph(tabBr);
				gui.displayGraph2(tabBr);
			}
			else gui.displayGraphMappings(tabBr);
		}
	}
	
	public void updateFeatures(String filePath, SchemaContentStructure scc) {
		XMLParser xmlp = new XMLParser();
		scc.clearLists();
		xmlp.startParsingFeatures(filePath, scc);
	}
	
	public static void main(String[] args) {
		XBenchMatch xbm = new XBenchMatch();
		gui = new GUI(xbm.matchersList, xbm);
		gui.initGUI();
	}

}
